var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var tsconfig = require('tsconfig');
var path_1 = require('path');
var fs_1 = require('fs');
var os_1 = require('os');
var make_error_1 = require('make-error');
var sourceMapSupport = require('source-map-support');
var extend = require('xtend');
var arrify = require('arrify');
var chalk = require('chalk');
exports.VERSION = '0.4.0';
exports.EXTENSIONS = ['.ts', '.tsx'];
function readConfig(cwd, ts) {
    var fileName = tsconfig.resolveSync(cwd);
    var config = fileName ? tsconfig.readFileSync(fileName, { filterDefinitions: true }) : {
        files: [],
        compilerOptions: {}
    };
    config.compilerOptions = extend({
        target: 'es5',
        module: 'commonjs'
    }, config.compilerOptions, {
        sourceMap: true,
        inlineSourceMap: false,
        inlineSources: false,
        declaration: false
    });
    return ts.parseConfigFile(config, ts.sys, fileName);
}
function register(opts) {
    var cwd = process.cwd();
    var options = extend({ getFile: getFile, getVersion: getVersion }, opts);
    var files = {};
    options.compiler = options.compiler || 'typescript';
    options.ignoreWarnings = arrify(options.ignoreWarnings).map(Number);
    var ts = require(options.compiler);
    var config = readConfig(options.project || cwd, ts);
    if (!options.disableWarnings && config.errors.length) {
        var error = new TypeScriptError(formatDiagnostics(config.errors, ts));
        console.error(error.formatMessage());
        process.exit(1);
    }
    var serviceHost = {
        getScriptFileNames: function () { return config.fileNames.concat(Object.keys(files)); },
        getScriptVersion: options.getVersion,
        getScriptSnapshot: function (fileName) {
            var contents = options.getFile(fileName);
            return contents ? ts.ScriptSnapshot.fromString(contents) : undefined;
        },
        getNewLine: function () { return os_1.EOL; },
        getCurrentDirectory: function () { return cwd; },
        getCompilationSettings: function () { return config.options; },
        getDefaultLibFileName: function (options) { return ts.getDefaultLibFilePath(config.options); }
    };
    var service = ts.createLanguageService(serviceHost);
    sourceMapSupport.install({
        retrieveFile: function (fileName) {
            if (files[fileName]) {
                return getOutput(fileName);
            }
        }
    });
    function getOutput(fileName) {
        var output = service.getEmitOutput(fileName);
        if (output.emitSkipped) {
            throw new TypeScriptError(path_1.relative(cwd, fileName) + ": Emit skipped");
        }
        var result = output.outputFiles[1].text;
        var sourceText = service.getSourceFile(fileName).text;
        var sourceMapText = output.outputFiles[0].text;
        var sourceMapFileName = output.outputFiles[0].name;
        var sourceMap = getSourceMap(sourceMapText, fileName, sourceText);
        var base64SourceMapText = new Buffer(sourceMap).toString('base64');
        return result
            .replace('//# sourceMappingURL=' + path_1.basename(sourceMapFileName), "//# sourceMappingURL=data:application/json;base64," + base64SourceMapText);
    }
    function addFileName(fileName) {
        files[fileName] = true;
    }
    function compile(fileName) {
        try {
            addFileName(fileName);
            validateDiagnostics(service, fileName, options, ts);
            return getOutput(fileName);
        }
        catch (error) {
            if (error.name === 'TypeScriptError' && !options.isEval) {
                console.error(error.formatMessage());
                process.exit(1);
            }
            throw error;
        }
    }
    function loader(m, fileName) {
        return m._compile(compile(fileName), fileName);
    }
    function getTypeInfo(fileName, position) {
        addFileName(fileName);
        var info = service.getQuickInfoAtPosition(fileName, position);
        var name = ts.displayPartsToString(info ? info.displayParts : []);
        var comment = ts.displayPartsToString(info ? info.documentation : []);
        return chalk.bold(name) + (comment ? "\n" + comment : '');
    }
    exports.EXTENSIONS.forEach(function (extension) {
        require.extensions[extension] = loader;
    });
    return { compile: compile, getTypeInfo: getTypeInfo };
}
exports.register = register;
function getVersion(fileName) {
    return String(fs_1.statSync(fileName).mtime.getTime());
}
exports.getVersion = getVersion;
function getFile(fileName) {
    try {
        return fs_1.readFileSync(fileName, 'utf8');
    }
    catch (err) { }
}
exports.getFile = getFile;
function validateDiagnostics(service, fileName, options, ts) {
    var diagnostics = ts.getPreEmitDiagnostics(service.getProgram())
        .filter(function (diagnostic) {
        return options.ignoreWarnings.indexOf(diagnostic.code) === -1;
    });
    if (!options.disableWarnings && diagnostics.length) {
        var message = formatDiagnostics(diagnostics, ts);
        throw new TypeScriptError(message);
    }
}
exports.validateDiagnostics = validateDiagnostics;
function formatDiagnostic(diagnostic, ts, cwd) {
    if (cwd === void 0) { cwd = '.'; }
    var message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
    if (diagnostic.file) {
        var path = path_1.relative(cwd, diagnostic.file.fileName);
        var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
        return path + " (" + (line + 1) + "," + (character + 1) + "): " + message + " (" + diagnostic.code + ")";
    }
    return message + " (" + diagnostic.code + ")";
}
exports.formatDiagnostic = formatDiagnostic;
function formatDiagnostics(diagnostics, ts) {
    return diagnostics.map(function (d) { return formatDiagnostic(d, ts); }).join(os_1.EOL);
}
function getSourceMap(map, fileName, code) {
    var sourceMap = JSON.parse(map);
    sourceMap.file = fileName;
    sourceMap.sources = [fileName];
    sourceMap.sourcesContent = [code];
    delete sourceMap.sourceRoot;
    return JSON.stringify(sourceMap);
}
exports.getSourceMap = getSourceMap;
var TypeScriptError = (function (_super) {
    __extends(TypeScriptError, _super);
    function TypeScriptError() {
        _super.apply(this, arguments);
        this.name = 'TypeScriptError';
    }
    TypeScriptError.prototype.formatMessage = function () {
        var boundary = chalk.grey('----------------------------------');
        return [
            boundary,
            chalk.red.bold('⨯ Unable to compile TypeScript'),
            '',
            this.message,
            boundary
        ].join(os_1.EOL);
    };
    return TypeScriptError;
})(make_error_1.BaseError);
exports.TypeScriptError = TypeScriptError;
//# sourceMappingURL=ts-node.js.map